-- 14-10-2021 TRK

-- KI tabeller - 12.9.5 - mss

patch_id fp_1295_211014


--************************************
-- Ny tabel: ki_pub_product                                         
--************************************



if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_product') and type = 'U')

create table dbo.ki_pub_product (
    pub_product_seq_no            numeric(9,0)    not null,
    task_seq_no                   numeric(9,0)    not null,
    public_date_dk                datetime        null,
    public_date_eu                datetime        null,
    valid_from_date               datetime        null,
    valid_to_date                 datetime        null,
    product_name_dk               varchar(256)    null,
    product_name_en               varchar(256)    null,
    product_description           varchar(800)    null,
    legislation_outcome_code      varchar(6)      null, -- ref. til ki_legislation_outcome.code
    legislation_code              varchar(6)      null, -- ref. til ki_legislation.code
    serial_number                 varchar(50)     null,
    batch_number                  varchar(50)     null,
    barcode                       varchar(50)     null,
    brand                         varchar(50)     null,
    compliance_type_id            varchar(6)      null, --- ref. til ki_pub_compliance_type.id
    ce_label_type_id              varchar(6)      null, --- ref. til ki_pub_ce_label_type.id
    serious_injust_type_id        varchar(6)      null, --- ref. til ki_pub_serious_injust_type.id
    voluntary_measure_type_id     varchar(6)      null, --- ref. til ki_pub_measure_type.id
    enforced_measure_type_id      varchar(6)      null, --- ref. til ki_pub_measure_type.id

    mst_source_type               varchar(50)     null,
    mst_reg                       varchar(1)      null, -- 'g'(grøn) el. 'r'(rød)
    mst_message                   varchar(500)    null, -- 'Produkt oprettet/opdateret' el. 'Kunne ikke oprette/opdatere produkt...'

    icsms_cid                     varchar(50)     null,
    icsms_piid                    bigint          null,
    icsms_reg                     varchar(1)      null, -- 'g'(grøn) el. 'r'(rød)
    icsms_message                 varchar(500)    null, -- 'Produkt oprettet' el. 'Kunne ikke oprette produkt...'

    created                       datetime        null default current_timestamp,
    created_by                    varchar(6)      null default current_user,
    changed                       datetime        null,
    changed_by                    varchar(6)      null,
    constraint pk_ki_pub_product primary key (pub_product_seq_no)
);

grant select on ki_pub_product to public;
grant select,insert,update,delete on ki_pub_product to public;


-- TRK: 2 Ekstra kolonner på ki_pub_product

if (select col_length('ki_pub_product','mst_timestamp')) is null 
alter table ki_pub_product
  add mst_timestamp datetime null;

if (select col_length('ki_pub_product','icsms_timestamp')) is null 
alter table ki_pub_product
  add icsms_timestamp datetime null;


--increment table 
insert into increment (table_name,column_name, last_used  ) Select 'ki_pub_product' ,'pub_product_seq_no', 100000  where (select count(*) from increment where table_name = 'ki_pub_product') = 0 ;

--************************************
-- Ny tabel: ki_pub_party                                         
--************************************


if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_party') and type = 'U')
CREATE TABLE dbo.ki_pub_party(
	ki_pub_party_id int IDENTITY (1, 1),
	pub_product_seq_no numeric(9,0) NOT NULL,
	party_type varchar(6) Not NULL ,
	party_name varchar(80) NOT NULL,
	addr varchar(150) NULL,
	addr_no varchar(10) NULL,
	postal_code  varchar(8) NULL,
	postal_city  varchar(50) NULL,
	country_code varchar(6) NULL,
	created datetime NULL default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
 Constraint pk_ki_pub_party primary key (ki_pub_party_id ) 
) ;

grant select,insert,update,delete on ki_pub_party to public;

--************************************
-- Ny tabel: ki_pub_note                                         
--************************************




if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_note') and type = 'U')

CREATE TABLE dbo.ki_pub_note(
	ki_pub_note_id int IDENTITY (1, 1),
	pub_product_seq_no numeric(9,0) NOT NULL,
	ki_pub_note varchar(500) NULL,
	ki_pub_link varchar(150) NULL,
	created datetime NULL default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
 Constraint pk_ki_pub_note primary key (ki_pub_note_id ) 
) ;
grant select,insert,update,delete on ki_pub_note to public;


--************************************
-- Ny tabel: ki_pub_files                                         
--************************************


if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_files') and type = 'U')

CREATE TABLE dbo.ki_pub_files(
	ki_pub_file_id int IDENTITY (1, 1),
	pub_product_seq_no numeric(9,0) NOT NULL,
	ki_pub_file_type  varchar(6) Not NULL,
	ki_pub_file_category  varchar(50)  NULL,
	ki_pub_file_source varchar(120)  NULL,
	ki_pub_file_name varchar(120)  NULL,
	ki_pub_file_path varchar(255) NULL,
	created datetime NULL default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	ki_pub_file_size int NULL,
 Constraint pk_ki_pub_files primary key (ki_pub_file_id ) 
) ;
grant select,insert,update,delete on ki_pub_files to public;


--**********************Nye type tables 
---ki_pub_serious_injust_type			Mest alvorlige skadestype
---ki_pub_compliance_type				produkttype 
---ki_pub_ce_label_type					CE Mærke
---ki_pub_measure_type				Frivillig/haandhaevet  foranstaltning
	
--************************************

if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_serious_injust_type') and type = 'U')

CREATE TABLE dbo.ki_pub_serious_injust_type(
	id varchar(6) NOT NULL,
	code_name varchar(60) NOT NULL,
	display_name varchar(256) NOT NULL,
	active varchar(1) NOT NULL default '1',
	created datetime NULL  default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	Constraint pk_ki_pub_serious_injust_type primary key (id) 
) ;
grant select on ki_pub_serious_injust_type to public;
grant select,insert,update,delete on ki_pub_serious_injust_type to sysmgr;

if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_compliance_type') and type = 'U')

CREATE TABLE dbo.ki_pub_compliance_type(
	id varchar(6) NOT NULL,
	code_name varchar(60) NOT NULL,
	display_name varchar(256) NOT NULL,
	active varchar(1) NOT NULL default '1',
	created datetime NULL  default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	Constraint pk_ki_pub_compliance_type primary key (id) 
) ;
grant select on ki_pub_compliance_type to public;
grant select,insert,update,delete on ki_pub_compliance_type to sysmgr;


if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_ce_label_type') and type = 'U')

CREATE TABLE dbo.ki_pub_ce_label_type(
	id varchar(6) NOT NULL,
	code_name varchar(60) NOT NULL,
	display_name varchar(256) NOT NULL,
	active varchar(1) NOT NULL default '1',
	created datetime NULL  default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	Constraint pk_ki_ce_label_type primary key (id) 
) ;
grant select on ki_pub_ce_label_type to public;
grant select,insert,update,delete on ki_pub_ce_label_type to sysmgr;

if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_measure_type') and type = 'U')

CREATE TABLE dbo.ki_pub_measure_type(
	id varchar(6) NOT NULL,
	code_name varchar(60) NOT NULL,
	display_name varchar(256) NOT NULL,
	active varchar(1) NOT NULL default '1',
	created datetime NULL  default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	Constraint pk_ki_pub_measure_type primary key (id) 
) ;
grant select on ki_pub_measure_type to public;
grant select,insert,update,delete on ki_pub_measure_type to sysmgr;

if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_product_category_type') and type = 'U')

CREATE TABLE dbo.ki_pub_product_category_type(
	id varchar(6) NOT NULL,
	category_id int NOT NULL,
	code_name varchar(60) NOT NULL,
	display_name varchar(256) NOT NULL,
	active varchar(1) NOT NULL default '1',
	scope	int NOT NULL default 1,
	created datetime NULL  default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	Constraint pk_ki_pub_product_category_type primary key (id) 
) ;
grant select on ki_pub_product_category_type to public;
grant select,insert,update,delete on ki_pub_product_category_type to sysmgr;



if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_product_reason_type') and type = 'U')

CREATE TABLE dbo.ki_pub_product_reason_type(
	id varchar(6) NOT NULL,
	code_name varchar(60) NOT NULL,
	reason varchar(1000) NOT NULL,
	active varchar(1) NOT NULL default '1',
	created datetime NULL  default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	Constraint pk_ki_pub_product_reason_type primary key (id) 
) ;
grant select on ki_pub_product_reason_type to public;
grant select,insert,update,delete on ki_pub_product_reason_type to sysmgr;



--******************************************************************************
-- x tables
--******************************************************************************

if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_product_x_category') and type = 'U')

CREATE TABLE dbo.ki_pub_product_x_category(
	pub_product_seq_no numeric(9,0) NOT NULL,
	pub_product_id varchar(6) NOT NULL,
	created datetime NULL default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
	ICSMS_product int NULL default 0,
 Constraint pk_ki_pub_product_x_category primary key (pub_product_seq_no, pub_product_id ) 
) ;
grant select,insert,update,delete on ki_pub_product_x_category to public;


if not exists (select 1 from sysobjects
    where id = object_id('ki_pub_product_x_reason') and type = 'U')

CREATE TABLE dbo.ki_pub_product_x_reason(
	pub_product_seq_no numeric(9,0) NOT NULL,
	pub_reason_id varchar(6) NOT NULL,
	created datetime NULL default current_timestamp,
	created_by varchar(6) NULL default current_user,
	changed datetime NULL,
	changed_by varchar(6) NULL,
 Constraint pk_ki_pub_product_x_reason primary key (pub_product_seq_no, pub_reason_id ) 
) ;
grant select,insert,update,delete on ki_pub_product_x_category to public;

 


--******************************************************************************
-- foreign_key
--******************************************************************************

drop_foreign_key on ki_pub_product ind_task
alter table ki_pub_product
  add constraint fk_ki_pub_product_ind_task foreign key (task_seq_no)
  references ind_task (task_seq_no);

drop_foreign_key on ki_pub_product ki_pub_compliance_type
alter table ki_pub_product
  add constraint fk_ki_pub_product_ki_pub_compliance_type foreign key (compliance_type_id)
  references ki_pub_compliance_type (id);

drop_foreign_key on ki_pub_product ki_pub_ce_label_type
alter table ki_pub_product
  add constraint fk_ki_pub_product_ki_pub_ce_label_type foreign key (ce_label_type_id)
  references ki_pub_ce_label_type (id);

drop_foreign_key on ki_pub_product ki_pub_serious_injust_type
alter table ki_pub_product
  add constraint fk_ki_pub_product_ki_pub_serious_injust_type foreign key (serious_injust_type_id)
  references ki_pub_serious_injust_type (id);

drop_foreign_key on ki_pub_product ki_pub_measure_type
alter table ki_pub_product
  add constraint fk_ki_pub_product_ki_pub_measure_type_1 foreign key (voluntary_measure_type_id)
  references ki_pub_measure_type (id);

--drop_foreign_key on ki_pub_product ki_pub_measure_type
alter table ki_pub_product
  add constraint fk_ki_pub_product_ki_pub_measure_type_2 foreign key (enforced_measure_type_id)
  references ki_pub_measure_type (id);


if not exists (select 1 from sysobjects
    where id = object_id('fk_ki_pub_party_ki_pub_product') and type = 'F')
alter table ki_pub_party
  add constraint fk_ki_pub_party_ki_pub_product foreign key (pub_product_seq_no)
  references ki_pub_product (pub_product_seq_no);

if not exists (select 1 from sysobjects
    where id = object_id('fk_ki_pub_note_ki_pub_product') and type = 'F')
alter table ki_pub_note
  add constraint fk_ki_pub_note_ki_pub_product foreign key (pub_product_seq_no)
  references ki_pub_product (pub_product_seq_no);

if not exists (select 1 from sysobjects
    where id = object_id('fk_ki_pub_files_ki_pub_product') and type = 'F')
alter table ki_pub_files
  add constraint fk_ki_pub_files_ki_pub_product foreign key (pub_product_seq_no)
  references ki_pub_product (pub_product_seq_no);

if not exists (select 1 from sysobjects
    where id = object_id('fk_ki_pub_product_x_category_ki_pub_product') and type = 'F')
alter table ki_pub_product_x_category
  add constraint fk_ki_pub_product_x_category_ki_pub_product foreign key (pub_product_seq_no)
  references ki_pub_product (pub_product_seq_no);

if not exists (select 1 from sysobjects
    where id = object_id('fk_ki_pub_product_x_category_ki_pub_product_category_type') and type = 'F')
alter table ki_pub_product_x_category
  add constraint fk_ki_pub_product_x_category_ki_pub_product_category_type foreign key (pub_product_id)
  references ki_pub_product_category_type (id);
  
if not exists (select 1 from sysobjects
    where id = object_id('fk_ki_pub_product_x_reason_ki_pub_product') and type = 'F')
alter table ki_pub_product_x_reason
  add constraint fk_ki_pub_product_x_reason_ki_pub_product foreign key (pub_product_seq_no)
  references ki_pub_product (pub_product_seq_no);

if not exists (select 1 from sysobjects
    where id = object_id('fk_ki_pub_product_x_reason_ki_pub_product_reason_type') and type = 'F')
alter table ki_pub_product_x_reason
  add constraint fk_ki_pub_product_x_reason_ki_pub_product_reason_type foreign key (pub_reason_id)
  references ki_pub_product_reason_type (id);
  

drop_foreign_key on ki_pub_product ki_legislation_outcome
alter table ki_pub_product
  add constraint fk_ki_pub_product_ki_legislation_outcome foreign key (legislation_outcome_code)
  references ki_legislation_outcome (code);

drop_foreign_key on ki_pub_product ki_legislation
alter table ki_pub_product
  add constraint fk_ki_pub_product_ki_legislation foreign key (legislation_code)
  references ki_legislation (code);